"""Formula Factory

Register a set of input handlers and spawn the correct formula
implementation.
"""
from ensemble.errors import FormulaError
import os.path


def _is_bundle(filename):
    """is_bundle(filename) -> boolean"""
    return os.path.isfile(filename) and \
        filename.endswith(".frml")


def get_formula_from_path(specification):
    """
    Given the specification of a formula (usually a pathname) map it
    to an implementation and create an instance of the proper type.
    """
    if _is_bundle(specification):
        from .bundle import FormulaBundle
        return FormulaBundle(specification)
    elif os.path.isdir(specification):
        from .directory import FormulaDirectory
        return FormulaDirectory(specification)

    raise FormulaError(specification, "Unable to process %s into a formula" % (
            specification))
