import os
import yaml

from ensemble.errors import FileNotFound
from ensemble.formula.provider import get_formula_from_path
from .errors import FormulaNotFound, FormulaError


class LocalFormulaRepository(object):
    """Formula repository in a local directory."""

    def __init__(self, path):
        self.path = path

    def __iter__(self):
        for dentry in os.listdir(self.path):
            dentry_path = os.path.join(self.path, dentry)
            try:
                yield get_formula_from_path(dentry_path)
            except FileNotFound:
                continue
            # there is a broken formula in the repo, but that
            # shouldn't stop us from continuing
            except (FormulaError, yaml.YAMLError):
                continue

    def find(self, name):
        """Find a formula with the given name.

        If multiple formulas are found with different versions, the most
        recent one (greatest revision) will be returned.
        """
        latest = None
        for formula in self:
            if formula.metadata.name == name:
                if (latest is None or
                    latest.metadata.revision < formula.metadata.revision):
                    latest = formula

        if latest is None:
            raise FormulaNotFound(self.path, name)

        return latest
