from ensemble.machine import ProviderMachine


class CobblerMachine(ProviderMachine):
    """Cobbler-specific provider machine implementation"""
    pass


class MachineProvider(object):

    def __init__(self, environment_name, config):
        self.environment_name = environment_name
        self.config = config

    def connect(self, share=False):
        """
        Connect to the zookeeper ensemble running in the machine provider.

        @param share: Requests sharing of the connection with other clients
            attempting to connect to the same provider, if that's feasible.

        returns an open C{txzookeeper.client.ZookeeperClient} and a
        C{ensemble.storage.connection.TunnelProtocol}
        """
        raise NotImplementedError()

    def list_machines(self):
        """
        List all machines running in the provider.
        """
        raise NotImplementedError()

    def start_machine(self, machine_data):
        """
        Start a machine in the provider.

        @param machine_data a dictionary of data to pass along to the newly
               launched machine.
        @type dict
        """
        raise NotImplementedError()

    def get_machine(self, provider_machine_id):
        """
        Retrieve a provider machine by provider machine id.
        """
        raise NotImplementedError()

    def bootstrap(self):
        """
        Bootstrap an ensemble server in the provider.
        """
        raise NotImplementedError()

    def shutdown(self):
        """
        Terminate any machine resources associated with the provider.
        """
        raise NotImplementedError()

    def shutdown_machine(self):
        """
        Stop a machine in the provider.
        """
        raise NotImplementedError()

    def save_state(self, state):
        """
        Save state to the provider.

        @param state
        @type dict
        """
        raise NotImplementedError()

    def load_state(self):
        """
        Load state from the provider, returns a dictionary.
        """
        raise NotImplementedError()

    def get_file_storage(self):
        """Retrieve the provider C{FileStorage} abstraction."""
        raise NotImplementedError()

    def get_serialization_data(self):
        """Return a dictionary serialization of the provider configuration.

        Additionally this extracts credential information from the environment.
        """
        raise NotImplementedError()

    def open_port(self, machine, port, protocol="tcp"):
        """Expose port to the environment.

        Approximate equivalent of ec2-authorize-group
        """
        raise NotImplementedError()

    def close_port(self, machine, port, protocol="tcp"):
        """Close opened port

        Approximate equivalent of ec2-revoke-group
        """
        raise NotImplementedError()

    def get_opened_ports(self, machine):
        """List ports currently exposed to the environment.

        Approximate equivalent of ec2-describe-group
        """
        raise NotImplementedError()


class FileStorage(object):

    def get(self, name):
        """Returns open file or fails with FileNotFound"""
        raise NotImplementedError()

    def put(self, remote_path, local_path):
        """Copies contents of local_path to remote_path"""
        raise NotImplementedError()
