from ensemble.errors import EnvironmentNotFound

from .utils import log


class Bootstrap(object):
    """Generic bootstrap operation class."""

    def __init__(self, provider):
        self._provider = provider

    def run(self):
        """Get an existing zookeeper, or launch a new one.

        @return: single-element list containing an appropriate
            ProviderMachine instance for the zookeeper.
        """
        machines = self._provider.get_zookeeper_machines()
        machines.addCallback(self._on_machines_found)
        machines.addErrback(self._on_no_machines_found)
        return machines

    def _on_machines_found(self, machines):
        log.info("Ensemble environment previously bootstrapped.")
        return machines

    def _on_no_machines_found(self, failure):
        failure.trap(EnvironmentNotFound)
        log.debug("Launching Ensemble bootstrap instance.")
        launch_class = self._provider.launch_machine_class
        launch = launch_class(self._provider, bootstrap=True)
        return launch.run({"machine-id": "0"})
