from yaml import safe_dump
from yaml import load
from txaws.s3.exception import S3Error


class EC2SaveState(object):

    def __init__(self, provider):
        self._provider = provider

    def run(self, state):
        """
        Save information on the EC2 ensemble instances to a persistent file
        on S3.
        """
        ensemble_bucket = self._provider.config["control-bucket"]
        d = self._provider.s3.get_bucket(ensemble_bucket)

        def on_no_bucket(failure):
            failure.trap(S3Error)
            return self._provider.s3.create_bucket(ensemble_bucket)

        def save_state(bucket):
            # serialize the instance ids and dns names to yaml
            serialized = safe_dump(state)
            return self._provider.s3.put_object(
                ensemble_bucket, "provider-state", serialized,
                content_type="text/plain")

        d.addErrback(on_no_bucket)
        d.addCallback(save_state)
        return d


class EC2LoadState(object):

    def __init__(self, provider):
        self._provider = provider

    def run(self):
        """
        Load and validate any saved state associated to the machine provider
        configuration. If its valid the resulting deferred returns the
        instance_id and dns name for the machine, else it returns None.
        """
        ensemble_bucket = self._provider.config["control-bucket"]

        def on_no_bucket(failure):
            failure.trap(S3Error)
            return None

        d = self._provider.s3.get_object(ensemble_bucket, "provider-state")
        d.addErrback(on_no_bucket)
        d.addCallback(self._on_state_retrieved)
        return d

    def _on_state_retrieved(self, content):
        if not content:
            return False
        state = load(content)
        if not state:
            return False
        return state
