import datetime

from txaws.ec2.model import Instance

from ensemble.lib.testing import TestCase
from ensemble.providers.ec2.machine import (
        EC2ProviderMachine, machine_from_instance)


class EC2ProviderMachineTest(TestCase):

    def test_machine_from_instance(self):
        instance = Instance(
            "i-foobar", "ignored",
            dns_name="public",
            private_dns_name="private",
            launch_time="2009-04-27T02:23:18.000Z")

        machine = machine_from_instance(instance)
        self.assertTrue(isinstance(machine, EC2ProviderMachine))
        self.assertEquals(machine.instance_id, instance.instance_id)
        self.assertEquals(machine.dns_name, instance.dns_name)
        self.assertEquals(machine.private_dns_name, instance.private_dns_name)
        self.assertEquals(machine.launch_time,
                          datetime.datetime(2009, 4, 27, 2, 23, 18))

    def test_machine_from_instance_no_launch_time(self):
        instance = Instance(
            "i-foobar", "ignored",
            dns_name="public",
            private_dns_name="private",
            launch_time="")

        machine = machine_from_instance(instance)
        self.assertTrue(isinstance(machine, EC2ProviderMachine))
        self.assertEquals(machine.instance_id, instance.instance_id)
        self.assertEquals(machine.dns_name, instance.dns_name)
        self.assertEquals(machine.private_dns_name, instance.private_dns_name)
        self.assertEquals(machine.launch_time, None)
